from flask import Flask, render_template

# Import our resources (blueprints)
from .IndexResource import index_api
from .SuffixResource import suffix_api
from .ReplicationResource import repl_api
from .LDAPOps import ldap_api
from .SchemaResource import schema_api
from .Authentication import auth_api
from .MonitorResource import monitor_api

from .ui.index import index_ui
from .ui.static import static_ui


rest389app = Flask(__name__)

# Register the blueprints
rest389app.register_blueprint(index_api)
rest389app.register_blueprint(suffix_api)
rest389app.register_blueprint(repl_api)
rest389app.register_blueprint(ldap_api)
rest389app.register_blueprint(schema_api)
rest389app.register_blueprint(auth_api)
rest389app.register_blueprint(monitor_api)

rest389app.register_blueprint(index_ui)
rest389app.register_blueprint(static_ui)


@rest389app.route('/apidoc')
def apidoc():
    """
    Must be last so that we get all the api bits registered.
    """

    urls = dict([
        (r.rule, {
            'docstr': rest389app.view_functions.get(r.endpoint).func_doc,
            'methods': r.methods,
            })
        for r in rest389app.url_map.iter_rules() if r.rule.startswith('/v1')
        ])  # End dict
    return render_template('apidoc.html', urls=urls)
